
# q-expansion of newform 357.2.i.b, downloaded from the LMFDB on 21 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 357
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[239, [1, 0]], [52, [0, -1]], [190, [1, 0]]]
aps_data = [[0, 0], [1, -1], [0, -4], [2, 1], [-4, 4], [1, 0], [-1, 1], [0, -1], [0, -2], [6, 0], [9, -9], [0, 11], [10, 0], [-7, 0], [0, -6], [6, -6], [-8, 8], [0, -6], [-3, 3], [-2, 0], [9, -9], [0, 3], [6, 0], [0, 0], [2, 0], [-14, 14], [0, 19], [0, 12], [1, -1], [12, 0], [1, 0], [0, -18], [-8, 8], [-9, 0], [0, -10], [-8, 8], [-10, 10], [0, -16], [14, 0], [0, -6], [6, -6], [7, 0], [0, 24], [-13, 13], [10, 0], [-16, 16], [-12, 0], [8, 0], [-4, 4], [0, -21], [0, 14], [-26, 0], [-6, 6], [-10, 0], [0, -2], [2, -2], [-22, 22], [0, -24], [5, -5], [26, 0], [25, -25], [-18, 0], [-13, 0], [-24, 24], [0, 19], [0, 20], [0, 13], [7, 0], [-2, 2], [-30, 0], [20, -20], [0, 0], [-13, 13], [0, 29], [-31, 0], [0, -12], [20, -20], [0, -1], [0, -6], [-5, 5], [-26, 0], [17, 0], [24, -24], [-41, 0], [0, 28], [0, -6], [-6, 0], [0, -37], [-2, 0], [15, 0], [0, 0], [-20, 20], [15, -15], [-20, 0], [0, 29], [6, 0], [0, 8], [-28, 28], [0, 13], [0, -19], [-20, 0], [-40, 40], [12, -12], [0, -4], [-15, 15], [21, -21], [-6, 0], [0, -4], [34, -34], [25, 0], [0, -25], [38, -38], [8, 0], [11, -11], [40, 0], [12, -12], [3, 0], [-24, 24], [0, 36], [6, 0], [25, -25], [41, 0], [0, -6], [4, -4], [0, -1], [38, 0], [0, -42], [0, -4], [-1, 0], [0, 23], [15, -15], [-4, 0], [0, -3], [6, 0], [0, -12], [-13, 0], [-46, 46], [44, -44], [-8, 0], [44, -44], [-4, 0], [0, -28], [-56, 56], [14, 0], [-21, 21], [-20, 0], [37, 0], [24, -24], [0, 20], [0, -24], [0, 42], [-32, 0], [-17, 0], [0, -10], [19, -19], [-48, 0], [0, 17], [0, -34], [43, 0], [32, -32], [0, 34], [8, 0], [-1, 0], [0, 34], [16, -16], [54, -54], [17, -17], [27, -27]]
